package net.hostsharing.hsadminng.hs.office.relation;

import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.Repository;

import jakarta.validation.constraints.NotNull;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

public interface HsOfficeRelationRbacRepository extends Repository<HsOfficeRelationRbacEntity, UUID> {

    Optional<HsOfficeRelationRbacEntity> findByUuid(UUID id);

    default List<HsOfficeRelationRbacEntity> findRelationRelatedToPersonUuidAndRelationType(@NotNull UUID personUuid, HsOfficeRelationType relationType) {
        return findRelationRelatedToPersonUuidAndRelationTypeString(personUuid, relationType == null ? null : relationType.toString());
    }

    @Query(value = """
            SELECT p.* FROM hs_office.relation_rv AS p
                WHERE p.anchorUuid = :personUuid OR p.holderUuid = :personUuid
            """, nativeQuery = true)
    List<HsOfficeRelationRbacEntity> findRelationRelatedToPersonUuid(@NotNull UUID personUuid);

    @Query(value = """
            SELECT p.* FROM hs_office.relation_rv AS p
                WHERE (:relationType IS NULL OR p.type = cast(:relationType AS hs_office.RelationType))
                    AND ( p.anchorUuid = :personUuid OR p.holderUuid = :personUuid)
            """, nativeQuery = true)
    List<HsOfficeRelationRbacEntity> findRelationRelatedToPersonUuidAndRelationTypeString(@NotNull UUID personUuid, String relationType);

    /**
     * Finds relations by a conjunction of optional criteria, including anchorPerson, holderPerson and contact data.
     *
     * @param personUuid the optional UUID of the anchorPerson or holderPerson
     * @param relationType the type of the relation
     * @param personData a lower-case string to match the persons tradeName, familyName or givenName (use '%' for wildcard)
     * @param contactData a lower-case string to match the contacts caption, postalAddress, emailAddresses or phoneNumbers (use '%' for wildcard)
     * @return a list of (accessible) relations which match all given criteria
     */
    @Query(value = """
            SELECT rel FROM HsOfficeRelationRbacEntity AS rel
                WHERE (:relationType IS NULL OR CAST(rel.type AS String) = CAST(:relationType AS String))
                    AND ( :personUuid IS NULL OR
                            rel.anchor.uuid = :personUuid OR rel.holder.uuid = :personUuid )
                    AND ( :personData IS NULL OR
                            lower(rel.anchor.tradeName) LIKE :personData OR lower(rel.holder.tradeName) LIKE :personData OR
                            lower(rel.anchor.familyName) LIKE :personData OR lower(rel.holder.familyName) LIKE :personData OR
                            lower(rel.anchor.givenName) LIKE :personData OR lower(rel.holder.givenName) LIKE :personData )
                    AND ( :contactData IS NULL OR
                            lower(rel.contact.caption) LIKE :contactData OR
                            lower(rel.contact.postalAddress) LIKE :contactData OR
                            lower(CAST(rel.contact.emailAddresses AS String)) LIKE :contactData OR
                            lower(CAST(rel.contact.phoneNumbers AS String)) LIKE :contactData )
            """)
    List<HsOfficeRelationRbacEntity> findRelationRelatedToPersonUuidRelationTypePersonAndContactData(
            UUID personUuid,
            HsOfficeRelationType relationType,
            String personData,
            String contactData);

    HsOfficeRelationRbacEntity save(final HsOfficeRelationRbacEntity entity);

    long count();

    int deleteByUuid(UUID uuid);
}
